/*
 * $Id: gstat.h,v 1.28 2002/02/28 21:55:49 spoel Exp $
 * 
 *                This source code is part of
 * 
 *                 G   R   O   M   A   C   S
 * 
 *          GROningen MAchine for Chemical Simulations
 * 
 *                        VERSION 3.1
 * Copyright (c) 1991-2001, University of Groningen, The Netherlands
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * If you want to redistribute modifications, please consider that
 * scientific software is very special. Version control is crucial -
 * bugs must be traceable. We will be happy to consider code for
 * inclusion in the official distribution, but derived work must not
 * be called official GROMACS. Details are found in the README & COPYING
 * files - if they are missing, get the official version at www.gromacs.org.
 * 
 * To help us fund GROMACS development, we humbly ask that you cite
 * the papers on the package - you can find them in the top README file.
 * 
 * For more info, check our website at http://www.gromacs.org
 * 
 * And Hey:
 * Getting the Right Output Means no Artefacts in Calculating Stuff
 */

#ifndef _gstat_h
#define _gstat_h

static char *SRCID_gstat_h = "$Id: gstat.h,v 1.28 2002/02/28 21:55:49 spoel Exp $";

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "typedefs.h"
#include "statutil.h"
#include "mshift.h"
#include "rmpbc.h"

#ifdef CPLUSPLUS
extern "C" {
#endif

/***********************************************
 *
 *     A U T O C O R R E L A T I O N
 *
 ***********************************************/

extern real LegendreP(real x,unsigned long m);

#define eacNormal (1<<0)
#define eacCos    (1<<1)
#define eacVector (1<<2)
#define eacRcross (1<<3  | eacVector)
#define eacP0     (1<<4  | eacVector)
#define eacP1     (1<<5  | eacVector)
#define eacP2     (1<<6  | eacVector)
#define eacP3     (1<<7  | eacVector)
#define eacP4     (1<<8  | eacVector)

enum {
  effnNONE, effnEXP1, effnEXP2, effnEXP3, effnVAC, 
  effnEXP5, effnEXP7, effnERREST, effnNR
};

extern int  nfp_ffn[effnNR];

extern char *s_ffn[effnNR+2];

extern char *longs_ffn[effnNR];

int sffn2effn(char **sffn);
/* Returns the ffn enum corresponding to the selected enum option in sffn */

extern t_pargs *add_acf_pargs(int *npargs,t_pargs *pa);
/* Add options for autocorr to the current set of options.
 * *npargs must be initialised to the number of elements in pa,
 * it will be incremented appropriately.
 */

extern void do_autocorr(char *fn,char *title,int nframes,int nitem,real **c1,
			real dt,unsigned long mode,bool bAver);
/* Calls low_do_autocorr (see below). After calling add_acf_pargs */

extern void low_do_autocorr(char *fn,char *title,
			    int  nframes,int nitem,int nout,real **c1,
			    real dt,unsigned long mode,int nrestart,
			    bool bAver,bool bFour,bool bNormalize,
			    bool bVerbose,real tbeginfit,real tendfit,
			    int nfitparm,int nskip);
/* 
 * do_autocorr calculates autocorrelation functions for many things.
 * It takes a 2 d array containing nitem arrays of length nframes
 * for each item the ACF is calculated.
 *
 * A number of "modes" exist for computation of the ACF
 *
 * if (mode == eacNormal) {
 *   C(t) = < X (tau) * X (tau+t) >
 * }
 * else if (mode == eacCos) {
 *   C(t) = < cos (X(tau) - X(tau+t)) >
 * }
 * else if (mode == eacVector) {
 *   C(t) = < X(tau) * X(tau+t)
 * }
 * else if (mode == eacP1) {
 *   C(t) = < cos (X(tau) * X(tau+t) >
 * }
 * else if (mode == eacP2) {
 *   C(t) = 1/2 * < 3 cos (X(tau) * X(tau+t) - 1 >
 * }
 * else if (mode == eacRcross) {
 *   C(t) = < ( X(tau) * X(tau+t) )^2 >
 * }
 *
 * For modes eacVector, eacP1, eacP2 and eacRcross the input should be 
 * 3 x nframes long, where each triplet is taken as a 3D vector
 *
 * For mode eacCos inputdata must be in radians, not degrees!
 *
 * Other parameters are:
 *
 * fn is output filename (.xvg) where the correlation function(s) are printed
 * title is the title in the output file
 * nframes is the number of frames in the time series
 * nitem is the number of items
 * c1 		is an array of dimension [ 0 .. nitem-1 ] [ 0 .. nframes-1 ]
 *    		on output, this array is filled with the correlation function
 *    		to reduce storage
 * nrestart     is the number of steps between restarts for direct ACFs 
 *              (i.e. without FFT) When set to 1 all points are used as
 *              time origin for averaging
 * dt 		is the time between frames
 * bAver 	If set, all ndih C(t) functions are averaged into a single 
 *       	C(t)
 * bFour      	If set, will use fast fourier transform (FFT) for evaluating
 *            	the ACF
 * bNormalize 	If set, all ACFs will be normalized to start at 0
 * nskip        Determines whether steps a re skipped in the output 
 */
 
 
/***********************************************
 *
 *     F I T   R O U T I N E S
 *
 ***********************************************/
extern void do_expfit(int ndata,real c1[],real dt,
		      real begintimefit,real endtimefit);

extern void expfit(int n, real x[], real y[], real Dy[], 
		   real *a, real *sa, 
		   real *b, real *sb);
/* This procedure fits y=exp(a+bx) for n (x,y) pairs to determine a and b.
 * The uncertainties in the y values must be in the vector Dy.
 * The standard deviations of a and b, sa and sb, are also calculated.
 *
 * Routine from Computers in physics, 7(3) (1993), p. 280-285.
 */


extern real fit_function(int eFitFn,real *parm,real x);
/* Returns the value of fit function eFitFn at x */

/* Use Levenberg-Marquardt method to fit to a nfitparm parameter exponential */
/* or to a transverse current autocorrelation function */
/* Or: "There is no KILL like OVERKILL", Dr. Ir. D. van der Spoel */
extern real do_lmfit(int ndata,real c1[],real sig[],real dt,real *x,
		     real begintimefit,real endtimefit,bool bVerbose,
		     int eFitFn,real fitparms[],int fix);
/* Returns integral.
 * If x == NULL, the timestep dt will be used to create a time axis.
 * fix fixes fit parameter i at it's starting value, when the i'th bit
 * of fix is set. 
 */

extern real print_and_integrate(FILE *fp,int n,real dt,
				real c[],real *fit,int nskip);
/* Integrate the data in c[] from 0 to n using trapezium rule.
 * If fp != NULL output is written to it
 * nskip determines whether all elements are written to the output file
 * (written when i % nskip == 0)
 * If fit != NULL the fit is also written.
 */
 
extern int get_acfnout(void);
/* Return the output length for the correlation function 
 * Works only AFTER do_auto_corr has been called!
 */

extern int get_acffitfn(void);
/* Return the fit function type.
 * Works only AFTER do_auto_corr has been called!
 */
 
 
/* Least squares method of computing statistics */
typedef struct {
  double yy,yx,xx,sx,sy;
  int    np;
} t_lsq;

extern void init_lsq(t_lsq *lsq);
extern void done_lsq(t_lsq *lsq);
extern void add_lsq(t_lsq *lsq,real x,real y);
extern void get_lsq_ab(t_lsq *lsq,real *a,real *b);
extern int  npoints_lsq(t_lsq *lsq);
extern real aver_lsq(t_lsq *lsq);
extern real sigma_lsq(t_lsq *lsq);
extern real error_lsq(t_lsq *lsq);

#ifdef CPLUSPLUS
	     }
#endif
 
#endif


